#delimit;
clear; set more off; set type double;
*ssc install estout, replace;

***************************************************
*** CODE TO REPLICATE MAIN RESULTS (TABLES 2-4) ***
***************************************************;

*** UPDATE THE GLOBAL path TO THE CORRECT FILEPATH ON YOUR COMPUTER;

global path "~/.../data_and_codes";
global st  $path/raw_data;
global res $path/results;
global cod $path/code;

*** VARIABLE LABELS;
#delimit;
global label_p      "Proportion correct";
global label_prof   "Proficiency rate";
global label_dp     "Derivative of proportion correct";
global label_d_prof "Derivative of expected proficiency";
global label_aw     "Density of ability distribution";
global label_prod   "Density x derivative";
global label_a      "Question discrimination, a_q";
global label_b      "Question difficulty, b_q";
global label_c      "Question guessability, c_q";
global label_Q      "Number of questions, Q";
global label_b_dev  "MSE between ability/difficulty";

#delimit;
global statelist ma ny nc tx il fl;

*** LOAD DATA AT STATE/YEAR/SUBJECT/GRADE/THETA LEVEL AND COLLAPSE TO STATE/YEAR/SUBJECT/GRADE LEVEL;

#delimit;
use $res/ma_results_bytheta, clear;
drop if _n>=1;

foreach st in $statelist {; append using $res/`st'_results_bytheta;};

*** USE LINEAR INTERPOLATION TO ESTIMATE VALUES OF dp, d_prof, AND aw AT VALUE OF THETA WHERE PROFICIENCY RATE (prof) IS 0.5;

global mlist d_prof aw b_dev;

replace b_dev=b_dev^2;

sort st yr subj grade theta;
foreach v in prof $mlist {;
 by st yr subj grade: gen `v'_margin=`v' if (prof[_n]<0.5 & prof[_n+1]>=0.5) | (prof[_n-1]<0.5 & prof[_n]>=0.5);
};

foreach st in $statelist {; foreach subj in math ela {;
 qui levelsof yr    if st=="`st'"           ; global yrs    `r(levels)'; foreach yr in $yrs    {;
 qui levelsof grade if st=="`st'" & yr==`yr'; global grades `r(levels)'; foreach g  in $grades {;
  foreach v in $mlist {; qui count if `v'~=. &            st=="`st'" & yr==`yr' & subj=="`subj'" & grade==`g'; if r(N)>0 {;
   qui reg     `v'_margin prof_margin                  if st=="`st'" & yr==`yr' & subj=="`subj'" & grade==`g';
   qui replace `v'_margin=_b[_cons]+.5*_b[prof_margin] if st=="`st'" & yr==`yr' & subj=="`subj'" & grade==`g';
  };};
};};};};

gen prod_margin=aw_margin*d_prof_margin;

bysort st yr grade: egen min_c=min(c);
bysort st yr grade: egen max_c=max(c);

compare min_c max_c;
replace a=. if min_c==max_c;
replace c=. if min_c==max_c;

*** COLLAPSE TO STATE/YEAR/SUBJECT/GRADE LEVEL;

collapse p prof theta scale dp d_prof_margin aw_margin prod_margin a b b_dev_margin c Q [aw=aw], by(st yr grade subj) fast;

rename d_prof_margin d_prof;
rename aw_margin     aw    ;
rename prod_margin   prod  ;
rename b_dev_margin  b_dev ;

***************************
*** TABLE 4 REGRESSIONS ***
***************************;

gen  math=subj=="math";
egen test=group(st yr grade);
gen  one =1;

global ylist prof p dp aw d_prof prod Q a b c b_dev;

#delimit;
foreach y in $ylist {; global y `y'; di "$y";
 eststo clear;
 qui count if $y~=.; qui summ $y if subj=="ela"; gen mean=r(mean);
 qui eststo:  reg mean one , nocons; drop mean;
 qui eststo: areg $y   math, robust absorb(test);
 qui eststo: areg $y   math if grade<=5   , robust absorb(test);
 qui eststo: areg $y   math if grade>=6   , robust absorb(test);
 qui eststo: areg $y   math if yr   < 2006, robust absorb(test);
 qui eststo: areg $y   math if yr   >=2006, robust absorb(test);
 esttab, rename(one math) keep(math) coeflabels(math "${label_$y}") varwidth(35) se b(2) star(* 0.10 ** 0.05 *** 0.01);
};

*********************************************************
*** COLLAPSE TO GRADE GROUP LEVEL AND MAKE TABLES 2-3 ***
*********************************************************;

#delimit;
gen min_grade=grade;
gen max_grade=grade;

replace grade=4 if grade<=5;
replace grade=8 if grade>=6;

preserve; replace st="all"; replace yr=2000 if yr<2006; replace yr=2006 if yr>=2006; save $st/toappend1, replace; restore;
preserve; replace st="all"; replace yr=9999; replace grade=99;                       save $st/toappend2, replace; restore;

forvalues i=1/2 {; append using $st/toappend`i'; erase $st/toappend`i'.dta;};

collapse p prof theta scale dp d_prof aw prod a b c Q (min) min_grade (max) max_grade, by(st yr grade subj) fast;

by st yr grade: gen ratio=prod[_n]/prod[_n-1];

drop a b c Q theta scale;

gen     state="Massachusetts"  if st=="ma" ;
replace state="New York"       if st=="ny" ;
replace state="North Carolina" if st=="nc" ;
replace state="Florida"        if st=="fl" ;
replace state="Texas"          if st=="tx" ;
replace state="Illinois"       if st=="il" ;
replace state="Grade 3-5 mean" if st=="all" & grade==4;
replace state="Grade 6-8 mean" if st=="all" & grade==8;
replace state="All exam mean"  if st=="all" & grade==99;

gen avg=st=="all";

gen     yr_grp=2000 if yr< 2006;
replace yr_grp=2006 if yr>=2006;
replace yr_grp=0    if yr==9999;

#delimit;
foreach v in p prof dp d_prof aw prod ratio {; gen temp=string(`v',"%20.2f"); drop `v'; rename temp `v';};
foreach v in yr                             {; gen temp=string(`v',"%20.0f"); drop `v'; rename temp `v';};

reshape wide p prof dp d_prof aw prod ratio, i(state yr_grp grade) j(subj) string;
drop ratioela; rename ratiomath ratio;
replace yr="" if avg==1;

sort yr_grp grade avg state;

#delimit;
gen     grades=string(min_grade,"%6.0f")                               if min_grade==max_grade & avg==0;
replace grades=string(min_grade,"%6.0f")+"-"+string(max_grade,"%6.0f") if min_grade~=max_grade & avg==0;
replace grades=string(min_grade,"%6.0f")+","+string(max_grade,"%6.0f") if min_grade~=max_grade & avg==0 & yr_grp==2000 & st=="il";

drop st yr_grp grade avg min_grade max_grade;

order state grades yr profmath profela pmath pela dpmath dpela awmath awela d_profmath d_profela prodmath prodela ratio;

browse;
