#delimit;
clear; set more off; set type double;

global state il;
global statename Illinois;

*** CODE TO COMPUTE RETURNS TO PREP FOR THE IL EXAM;

*****************
*** 2008 EXAM ***
*****************;

global N $obs; *** NUMBER OF TEST TAKERS AT EACH SKILL LEVEL;
global D 1.7 ; *** NORMALIZATION CONSTANT (FROM TECHNICAL REPORT);

*** USE THE RAW-TO-SCALE CONVERSION DATA TO CREATE EXAM TEMPLATES;

use "$st/il_raw_to_scale", clear;
drop if raw==0;
drop scale prof_level;
rename raw item;

*** CREATE A DATASET WITH $N TEST TAKERS FOR EACH EXAM AND EACH SKILL LEVEL FROM -5, 4.9, ..., 5;
gen i=.; save "$st/toappend", replace; replace i=1;
forvalues i=2/$N {; qui append using "$st/toappend"; qui replace i=`i' if i==.;};

gen theta=.; save "$st/toappend", replace; replace theta=-5;
forvalues t=-49(1)50 {; qui append using "$st/toappend"; qui replace theta=`t'/10 if theta==.;}; erase "$st/toappend.dta"; compress;
sort theta yr subj grade i item; compress;

*** MERGE IN THE QUESTION DIFFICULTY DATA - USING DISTRIBUTION OF IRT PARAMETERS;

sort  yr subj grade item;
merge yr subj grade using "$st/il_irt_difficulty";
tab _merge; tab grade _merge; drop if _merge==2; drop _merge;

*** APPROXIMATE THE DIFFICULTY PARAMETER BY DRAWING A NORMAL DISTRIBUTION OF THE b VALUES AROUND THE GIVEN MEAN AND SD.;
*** RESTRICT VALUES TO WITHIN +/- 2.57 SDs OF THE MEAN; 

gen rand1=runiform()*0.99+.005 ;
gen  b=invnormal(rand1)*b_sd*1.05+b_mean; 
summ b;

gen rand2=runiform()*0.99+.005;
gen  a=invnormal(rand2)*a_sd+a_mean; 
summ a;

replace b=invnormal(rand1)*b_sd+b_mean;
replace a=a_mean;

gen c=c_mean;

drop a_mean a_sd a_min a_max b_mean b_sd b_min b_max c_mean c_sd c_min c_max rand1 rand2;

*** DRAW ITEM RESPONSES (u)
*** - NOTE THAT CA ONLY REPORTS P-VALUES BY ITEM. 
*** - WE APPROXIMATE THE DIFFICULTY PARAMETER BY SOLVING FOR b FOR AN AVERAGE TEST TAKER (theta=0) AND USING THE P-VALUE;
#delimit; 
gen p_correct=c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen rand     =runiform();
gen byte u   =rand<p_correct;
drop rand p_correct;

*** COMPUTE RAW SCORE (raw) AND MERGE IN SCALE SCORES (scale);

bysort yr subj grade theta i: egen byte raw=total(u);
sort   yr subj grade raw; merge yr subj grade raw using "$st/il_raw_to_scale"; tab _merge; tab raw if _merge==2; drop if _merge==2; drop _merge;

*** COMPUTE SAMPLE WEIGHTS (aw) TO MATCH THE ACTUAL SCORE DISTRIBUTION;
#delimit;
gen phi=normalden(theta);
sort   yr subj grade prof_level; merge yr subj grade prof_level using "$st/il_scale_distribution"; tab _merge; tab scale if _merge==2; drop if _merge==2; replace dist=0 if _merge==1; drop _merge;
bysort yr subj grade prof_level: egen tot_scale=total(phi);
by     yr subj grade           : egen tot      =total(phi); 
gen prior=tot_scale/tot; drop tot tot_scale;
gen aw=phi*dist/prior; drop dist prior;

*** COMPUTE SEVERAL DIFFERENT TEST SCORE TRANSFORMATIONS:
*** - tau   = EXPECTED ABILITY GIVEN RAW SCORE IN REFERENCE POPULATION (WHERE theta~N(0,1))
*** - prof  = INDICATOR FOR PROFICIENT
*** - index = PROFICIENCY INDEX: 1 IF PROFICIENT, R_i/R* IF NOT (WHERE R* IS THE RAW PROFICIENCY THRESHOLD);

#delimit;
gen prod=phi*theta;
bysort yr subj grade raw: egen tot_phi =total(phi);
by     yr subj grade raw: egen tot_prod=total(prod);
gen tau=tot_prod/tot_phi; drop tot_* prod;

gen prof=prof_level>=3;

gen temp=raw if prof==1; bysort yr subj grade: egen raw_prof=min(temp); drop temp;
gen index=min(raw/raw_prof,1);

*** COMPUTE RETURNS TO A CORRECT ANSWER TO EACH QUESTION FOR THE DIFFERENT TEST SCORE TRANSFORMATIONS (*_return)
*** - raw0 = RAW SCORE WHEN QUESTION q IS ANSWERED INCORRECTLY (SIMILARLY FOR scale0, tau0, prof0, index0)
*** - raw1 = RAW SCORE WHEN QUESTION q IS ANSWERED   CORRECTLY (SIMILARLY FOR scale1, tau1, prof1, index1)
*** - NOTE THAT tau1 OR tau0 MAY BE MISSING IF NO ONE EARNED A PARTICULAR RAW SCORE VALUE---THIS WON'T REALLY AFFECT OUR CALCULATIONS SINCE THESE ARE RARE OUTCOMES;
#delimit;
gen raw0=raw -u; 
gen raw1=raw0+1;

rename raw raw_actual; rename scale scale_actual;
rename raw0 raw ; merge m:1 yr subj grade raw using "$st/il_raw_to_scale"; drop if _merge==2; drop _merge;
rename raw  raw0; rename scale scale0;
rename raw1 raw ; merge m:1 yr subj grade raw using "$st/il_raw_to_scale"; drop if _merge==2; drop _merge;
rename raw  raw1; rename scale scale1;
rename raw_actual raw; rename scale_actual scale;

gen temp=tau if raw==raw0; bysort yr subj grade raw0: egen tau0=mean(temp); drop temp;
gen temp=tau if raw==raw1; bysort yr subj grade raw1: egen tau1=mean(temp); drop temp;

gen prof0=raw0>=raw_prof;
gen prof1=raw1>=raw_prof;

gen index0=min(raw0/raw_prof,1);
gen index1=min(raw1/raw_prof,1);

gen scale_return=scale1-scale0;
gen tau_return  =tau1  -tau0  ;
gen prof_return =prof1 -prof0 ;
gen index_return=index1-index0;

*** COMPUTE DERIVATIVE OF PROBABILITY OF A CORRECT ANSWER FOR EACH QUESTION (dp)
*** COMPUTE THE TOTAL SCORE RETURN TO AN INCREASE ABILITY FOR EACH TEST SCORE TRANSFORMATION (d_*)
*** NORMALIZE THE SCORE RETURNS TO REFLECT 1 CORRECT ANSWER ON A 50-QUESTION EXAM;

gen p =c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen dp=$D*a*(1-c)*exp(-$D*a*(theta-b))/(1+exp(-$D*a*(theta-b)))^2;
gen I =dp^2/(p*(1-p));

qui compress;
save $st/il2008_full, replace;

*****************
*** 2000 EXAM ***
*****************;

*** CODE TO COMPUTE RETURNS TO PREP FOR THE IL 2000 EXAM;

#delimit;
global N $obs; *** NUMBER OF TEST TAKERS AT EACH SKILL LEVEL;
global D 1   ; *** NORMALIZATION CONSTANT (FROM TECHNICAL REPORT);

*** USE THE QUESTION DIFFICULTY DATA AND RESHAPE ONE OBSERVATION PER SUBJECT/GRADE;

use $st/il2000_irt_difficulty, clear;
summ item;
global Q=r(max);
reshape wide a b c, i(yr subj grade) j(item);
order yr subj grade a* b* c*;

*** CREATE A DATASET WITH $N TEST TAKERS FOR EACH EXAM AND EACH SKILL LEVEL FROM -5, 4.9, ..., 5;

gen i=.;
save $st/toappend, replace;
replace i=1;
forvalues i=2/$N {; qui append using $st/toappend; qui replace i=`i' if i==.;};

gen theta=.;
save $st/toappend, replace;
replace theta=-5;
forvalues t=-49(1)50 {;	qui append using $st/toappend; qui replace theta=`t'/10 if theta==.;};
erase $st/toappend.dta;
sort  yr subj grade theta i;
order yr subj grade theta i;
compress;

*** DRAW ITEM RESPONSES (u);

forvalues q=1/$Q {; gen byte u`q'=runiform()<c`q'+(1-c`q')/(1+exp(-$D*a`q'*(theta-b`q'))) if b`q'~=.;};

*** COMPUTE THE VARIABLE tau, WHICH IS THE INDIVIDUAL'S TEST SCORE IN THETA UNITS
***  - tau IS EQUAL TO THE INDIVIDUAL'S EXPECTED ABILITY GIVEN THEIR RESPONSE VECTOR (u1,...,uQ)
***  - COMPUTE THIS BY INTEGRATING OVER VALUES OF x FROM -5 TO 5 FOR AN ASSUMED DISTRIBUTION OF ABILITY x~N(0,1)
*** DO THIS FOR:
***  - THE ACTUAL RESPONSE VECTOR (u) -> tau
***  - THE RESPONSE VECTOR IN WHICH QUESTION q IS INCORRECT (FOR EACH Q) -> tau0`q'
***  - THE RESPONSE VECTOR IN WHICH QUESTION q IS   CORRECT (FOR EACH Q) -> tau1`q';

#delimit;
forvalues q=1/$Q {; gen Pru1_q`q'=0 if b`q'~=.; gen tau_u1_q`q'=0 if b`q'~=.;
                    gen Pru0_q`q'=0 if b`q'~=.; gen tau_u0_q`q'=0 if b`q'~=.;};
                    gen Pru=0                 ; gen tau_u      =0           ;

#delimit;
forvalues x=-50(1)50 {; global x=`x'/10; di "Ability $x";
 gen phi  =normalden($x);
 gen Pru_x=1;
 forvalues q=1/$Q {; qui replace Pru_x=Pru_x*(c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q') if b`q'~=.;};
 qui replace Pru  =Pru  +   phi*Pru_x;
 qui replace tau_u=tau_u+$x*phi*Pru_x;

 forvalues q=1/$Q {;
  qui replace Pru1_q`q'  =Pru1_q`q'  +   phi*Pru_x*(  c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))/((c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')) if b`q'~=. & (c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')~=0;
  qui replace tau_u1_q`q'=tau_u1_q`q'+$x*phi*Pru_x*(  c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))/((c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')) if b`q'~=. & (c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')~=0;
  qui replace Pru0_q`q'  =Pru0_q`q'  +   phi*Pru_x*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))/((c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')) if b`q'~=. & (c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')~=0;
  qui replace tau_u0_q`q'=tau_u0_q`q'+$x*phi*Pru_x*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))/((c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')) if b`q'~=. & (c`q'+(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(u`q')*(1-c`q'-(1-c`q')/(1+exp(-$D*a`q'*($x-b`q'))))^(1-u`q')~=0;
 };

 drop phi Pru_x;
};

gen tau=tau_u/Pru;

#delimit;
forvalues q=1/$Q {; di "`q'";
 qui gen tau1`q'=tau_u1_q`q'/Pru1_q`q' if b`q'~=.;
 qui gen tau0`q'=tau_u0_q`q'/Pru0_q`q' if b`q'~=.;
 qui gen dp`q'  =$D*a`q'*(1-c`q')*exp(-$D*a`q'*(theta-b`q'))/(1+exp(-$D*a`q'*(theta-b`q')))^2 if b`q'~=.;
 qui gen dtau`q'=(tau1`q'-tau0`q')*dp`q' if b`q'~=.;
};

qui compress;
save $st/afterloop, replace;

#delimit;
use $st/afterloop, clear;
drop    tau_u Pru tau_u0_q* tau_u1_q* Pru0_q* Pru1_q* dp* dtau*;

*** CONVERT TEST SCORE FROM THETA UNITS (tau) TO SCALE SCORE UNITS (scale);
#delimit;
merge m:1 yr subj grade using $st/il2000_raw_to_scale; drop if _merge==2; drop _merge;

gen scale=A*tau+B;

gen byte prof_level=1;
replace  prof_level=2 if scale>=prof_cut2;
replace  prof_level=3 if scale>=prof_cut3;
replace  prof_level=4 if scale>=prof_cut4;
tab      prof_level, missing;

*** COMPUTE SAMPLE WEIGHTS (aw) TO MATCH THE ACTUAL SCORE DISTRIBUTION;

#delimit;
gen phi=normalden(theta);
merge m:1 yr subj grade prof_level using $st/il2000_scale_distribution; drop if _merge==2;
replace dist=0 if _merge==1; drop _merge;

bysort yr subj grade prof_level: egen tot_scale=total(phi);
by     yr subj grade           : egen tot      =total(phi);
gen prior=tot_scale/tot; drop tot tot_scale;
gen aw=phi*dist/prior; drop dist prior phi;

*** RESHAPE TO AN INDIVIDUAL/QUESTION-LEVEL DATASET;

#delimit;
compress;
save $st/beforeappend, replace;

#delimit;
forvalues q=1/$Q {; global q `q'; di "$q";
 use $st/beforeappend, clear;
 rename a$q tempa; rename b$q tempb; rename c$q tempc; rename u$q tempu; rename tau0$q temptau0; rename tau1$q temptau1;
 forvalues i=1/$Q {; capture drop a`i' b`i' c`i' u`i' tau1`i' tau0`i';};
 renpfix temp;
 gen item=$q;
 qui save $st/toappend$q, replace;
};

#delimit;
use $st/toappend1, clear; forvalues q=2/$Q {; append using $st/toappend`q';};
drop if b==.;

#delimit;
forvalues q=1/$Q {; erase $st/toappend`q'.dta;};
erase $st/beforeappend.dta;

qui compress;
save $st/longdata, replace;

*** COMPUTE SEVERAL DIFFERENT TEST SCORE TRANSFORMATIONS:
*** - raw   = NUMBER OF QUESTIONS CORRECT
*** - prof  = INDICATOR FOR PROFICIENT
*** - index = PROFICIENCY INDEX: 1 IF PROFICIENT, R_i/R* IF NOT (WHERE R* IS THE RAW PROFICIENCY "THRESHOLD");

#delimit;
use $st/longdata, clear;
bysort yr subj grade theta i item: gen dup=_N; tab dup; drop dup;

by yr subj grade theta i: egen raw=total(u);
tab  raw;

gen prof=prof_level>=3;

gen temp=raw if prof==1; tab temp;
by yr subj grade: egen raw_prof=min(temp); drop temp;
gen index=min(raw/raw_prof,1);

*** COMPUTE RETURNS TO A CORRECT ANSWER TO EACH QUESTION FOR THE DIFFERENT TEST SCORE TRANSFORMATIONS (*_return)
*** - raw0 = RAW SCORE WHEN QUESTION q IS ANSWERED INCORRECTLY (SIMILARLY FOR scale0, tau0, prof0, index0)
*** - raw1 = RAW SCORE WHEN QUESTION q IS ANSWERED   CORRECTLY (SIMILARLY FOR scale1, tau1, prof1, index1);
#delimit;
gen raw0=raw -u;
gen raw1=raw0+1;

#delimit;
gen scale0=A*tau0+B;
gen scale1=A*tau1+B;

#delimit;
forvalues i=0/1 {;
 gen byte prof_level`i'=1;
 replace  prof_level`i'=2 if scale`i'>=prof_cut2;
 replace  prof_level`i'=3 if scale`i'>=prof_cut3;
 replace  prof_level`i'=4 if scale`i'>=prof_cut4;
};

gen prof0=prof_level0>=3;
gen prof1=prof_level1>=3;

gen index0=min(raw0/raw_prof,1);
gen index1=min(raw1/raw_prof,1);

gen scale_return=scale1-scale0;
gen tau_return  =tau1  -tau0  ;
gen prof_return =prof1 -prof0 ;
gen index_return=index1-index0;

*** COMPUTE DERIVATIVE OF PROBABILITY OF A CORRECT ANSWER FOR EACH QUESTION (dp)
*** COMPUTE THE TOTAL SCORE RETURN TO AN INCREASE ABILITY FOR EACH TEST SCORE TRANSFORMATION (d_*)
*** NORMALIZE THE SCORE RETURNS TO REFLECT 1 CORRECT ANSWER ON A 50-QUESTION EXAM;
#delimit;
gen p =c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen dp=$D*a*(1-c)*exp(-$D*a*(theta-b))/(1+exp(-$D*a*(theta-b)))^2;
gen I =dp^2/(p*(1-p));

*** APPEND 2008 EXAM AND COLLAPSE;

append using $st/il2008_full;

do $cod/collapse-data.do;

erase $st/il2008_full.dta;
erase $st/afterloop.dta
erase $st/longdata.dta

