#delimit;
clear; set more off; set type double;

global state ma;
global statename Massachusetts;  

*** CODE TO COMPUTE RETURNS TO PREP FOR THE MA EXAM;

global N $obs ; *** NUMBER OF TEST TAKERS AT EACH SKILL LEVEL;
global D 1.701; *** NORMALIZATION CONSTANT (FROM TECHNICAL REPORT);

*** USE THE RAW-TO-SCALE CONVERSION DATA TO CREATE EXAM TEMPLATES;

use $st/ma_raw_to_scale, clear;
drop if raw==0;
drop scale prof_level;
rename raw item;

*** CREATE A DATASET WITH $N TEST TAKERS FOR EACH EXAM AND EACH SKILL LEVEL FROM -5, 4.9, ..., 5;

gen i=.; save $st/toappend, replace; replace i=1;
forvalues i=2/$N {; qui append using $st/toappend; qui replace i=`i' if i==.;};

gen theta=.; save $st/toappend, replace; replace theta=-5;
forvalues t=-49(1)50 {; qui append using $st/toappend; qui replace theta=`t'/10 if theta==.;}; erase $st/toappend.dta; compress;
sort theta yr subj grade i item; compress;

*** MERGE IN THE QUESTION DIFFICULTY DATA;

merge m:1 yr subj grade item using $st/ma_irt_difficulty;
tab _merge; tab grade _merge; drop if _merge==2; drop _merge;

*** DRAW ITEM RESPONSES (u);

gen p_correct=c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen rand     =runiform();
gen byte u   =rand<p_correct;
drop rand p_correct;

*** COMPUTE RAW SCORE (raw) AND MERGE IN SCALE SCORES (scale);

bysort yr subj grade theta i: egen byte raw=total(u);
merge m:1 yr subj grade raw using $st/ma_raw_to_scale; tab _merge; tab raw if _merge==2; drop if _merge==2; drop _merge;

*** COMPUTE SAMPLE WEIGHTS (aw) TO MATCH THE ACTUAL SCORE DISTRIBUTION
*** FOR 2000 AND GRADE 3 2006 WE ONLY HAVE THE SCALE SCORE DISTRIBUTION AT THE PROFICIENCY LEVEL;

gen     scale_orig=scale;
replace scale=. if yr==2000;
replace scale=. if grade==3;

gen phi=normalden(theta);
merge m:1 yr subj grade scale prof_level using $st/ma_scale_distribution; tab scale if _merge==2; drop if _merge==2; replace dist=0 if _merge==1; drop _merge;
bysort    yr subj grade scale prof_level: egen tot_scale=total(phi);
by        yr subj grade                 : egen tot      =total(phi);
gen prior=tot_scale/tot; drop tot tot_scale;
gen aw=phi*dist/prior; drop dist prior;
replace scale=scale_orig; drop scale_orig;

*** COMPUTE SEVERAL DIFFERENT TEST SCORE TRANSFORMATIONS:
*** - tau   = EXPECTED ABILITY GIVEN RAW SCORE IN REFERENCE POPULATION (WHERE theta~N(0,1))
*** - prof  = INDICATOR FOR PROFICIENT
*** - index = PROFICIENCY INDEX: 1 IF PROFICIENT, R_i/R* IF NOT (WHERE R* IS THE RAW PROFICIENCY THRESHOLD);

#delimit;
gen prod=phi*theta;
bysort yr subj grade raw: egen tot_phi =total(phi);
by     yr subj grade raw: egen tot_prod=total(prod);
gen tau=tot_prod/tot_phi; drop tot_* prod;

gen prof=prof_level>=3;

gen temp=raw if prof==1; bysort yr subj grade: egen raw_prof=min(temp); drop temp;
gen index=min(raw/raw_prof,1);

*** COMPUTE RETURNS TO A CORRECT ANSWER TO EACH QUESTION FOR THE DIFFERENT TEST SCORE TRANSFORMATIONS (*_return)
*** - raw0 = RAW SCORE WHEN QUESTION q IS ANSWERED INCORRECTLY (SIMILARLY FOR scale0, tau0, prof0, index0)
*** - raw1 = RAW SCORE WHEN QUESTION q IS ANSWERED   CORRECTLY (SIMILARLY FOR scale1, tau1, prof1, index1)
*** - NOTE THAT tau1 OR tau0 MAY BE MISSING IF NO ONE EARNED A PARTICULAR RAW SCORE VALUE---THIS WON'T REALLY AFFECT OUR CALCULATIONS SINCE THESE ARE RARE OUTCOMES;
#delimit;
gen raw0=raw -u;
gen raw1=raw0+1;

rename raw raw_actual; rename scale scale_actual;
rename raw0 raw ; merge m:1 yr subj grade raw using $st/ma_raw_to_scale; drop if _merge==2; drop _merge;
rename raw  raw0; rename scale scale0;
rename raw1 raw ; merge m:1 yr subj grade raw using $st/ma_raw_to_scale; drop if _merge==2; drop _merge;
rename raw  raw1; rename scale scale1;
rename raw_actual raw; rename scale_actual scale;

gen temp=tau if raw==raw0; bysort yr subj grade raw0: egen tau0=mean(temp); drop temp;
gen temp=tau if raw==raw1; bysort yr subj grade raw1: egen tau1=mean(temp); drop temp;

gen prof0=raw0>=raw_prof;
gen prof1=raw1>=raw_prof;

gen index0=min(raw0/raw_prof,1);
gen index1=min(raw1/raw_prof,1);

gen scale_return=scale1-scale0;
gen tau_return  =tau1  -tau0  ;
gen prof_return =prof1 -prof0 ;
gen index_return=index1-index0;

*** COMPUTE DERIVATIVE OF PROBABILITY OF A CORRECT ANSWER FOR EACH QUESTION (dp)
*** COMPUTE THE TOTAL SCORE RETURN TO AN INCREASE ABILITY FOR EACH TEST SCORE TRANSFORMATION (d_*)
*** NORMALIZE THE SCORE RETURNS TO REFLECT 1 CORRECT ANSWER ON A 50-QUESTION EXAM;

gen p =c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen dp=$D*a*(1-c)*exp(-$D*a*(theta-b))/(1+exp(-$D*a*(theta-b)))^2;
gen I =dp^2/(p*(1-p));

do $cod/collapse-data.do;
