#delimit;
clear; set more off; set type double;

global state nc;
global statename North Carolina;  

*** CODE TO COMPUTE RETURNS TO PREP FOR THE NC EXAM;

global N $obs; *** NUMBER OF TEST TAKERS AT EACH SKILL LEVEL;
global D 1.7 ; *** NORMALIZATION CONSTANT (FROM TECHNICAL REPORT);

*** USE THE RAW-TO-SCALE CONVERSION DATA TO CREATE EXAM TEMPLATES;

use $st/nc_raw_to_scale, clear;
summ Q; global Q=r(max);
gen item=1;
forvalues i=2/$Q {; append using $st/nc_raw_to_scale; replace item=`i' if item==.;};

drop if item>Q;
bysort yr subj grade item: gen dup=_N; tab dup; drop dup;

*** CREATE A DATASET WITH $N TEST TAKERS FOR EACH EXAM AND EACH SKILL LEVEL FROM -5, 4.9, ..., 5;
#delimit;
gen i=.; save $st/toappend, replace; replace i=1;
forvalues i=2/$N {; qui append using $st/toappend; qui replace i=`i' if i==.;};

gen theta=.; save $st/toappend, replace; replace theta=-5;
forvalues t=-49(1)50 {; qui append using $st/toappend; qui replace theta=`t'/10 if theta==.;}; erase $st/toappend.dta; compress;
sort theta yr subj grade i item; compress;

*** MERGE IN THE QUESTION DIFFICULTY DATA
*** FOR NC, WE ONLY OBSERVE THE MEAN (a,b,c) VALUES
*** SET a AND c EQUAL TO THEIR MEAN VALUES FOR ALL QUESTIONS
*** DRAW A NORMAL DISTRIBUTION OF b VALUES AROUND THE GIVEN MEAN
*** - ASSUME A STANDARD DEVIATION OF 1 AND RESTRICT TO VALUES WITHIN +/- 2.57 SDs OF THE MEAN;
#delimit;
sort  yr subj grade;
merge yr subj grade using $st/nc_irt_difficulty;
tab _merge; tab grade _merge; drop if _merge==2; drop _merge;

gen a=a_mean;
gen c=c_mean;

gen b_sd=1;
gen rand=runiform()*0.99+0.005;

gen b=invnormal(rand)*b_sd+b_mean;

drop a_mean b_mean c_mean b_sd pvalue_mean rand;

*** DRAW ITEM RESPONSES (u);
#delimit;
gen p_correct=c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen rand     =runiform();
gen byte u   =rand<p_correct;
drop rand p_correct;

*** COMPUTE RAW SCORE (raw);
#delimit;
bysort yr subj grade theta i: egen byte raw=total(u);

*** COMPUTE tau = EXPECTED ABILITY GIVEN RAW SCORE IN REFERENCE POPULATION (WHERE theta~N(0,1));

#delimit;
gen phi=normalden(theta);
gen prod=phi*theta;
bysort yr subj grade raw: egen tot_phi =total(phi);
by     yr subj grade raw: egen tot_prod=total(prod);
gen tau=tot_prod/tot_phi; drop tot_* prod;

*** GENERATE SCALE SCORES AND PROFICIENCY LEVELS;

#delimit;
gen scale=A*tau+B;

gen byte prof_level=1;
replace  prof_level=2 if scale>=prof_cut2;
replace  prof_level=3 if scale>=prof_cut3;
replace  prof_level=4 if scale>=prof_cut4;
tab      prof_level, missing;

*** COMPUTE SAMPLE WEIGHTS (aw) TO MATCH THE ACTUAL SCORE DISTRIBUTION;
#delimit;
merge m:1 yr subj grade prof_level using $st/nc_scale_distribution; tab scale if _merge==2; drop if _merge==2; replace dist=0 if _merge==1; drop _merge;
bysort    yr subj grade prof_level: egen tot_scale=total(phi);
by        yr subj grade           : egen tot      =total(phi);
gen prior=tot_scale/tot; drop tot tot_scale;
gen aw=phi*dist/prior; drop dist prior;

*** COMPUTE SEVERAL DIFFERENT TEST SCORE TRANSFORMATIONS:
*** - tau   = EXPECTED ABILITY GIVEN RAW SCORE IN REFERENCE POPULATION (WHERE theta~N(0,1))
*** - prof  = INDICATOR FOR PROFICIENT
*** - index = PROFICIENCY INDEX: 1 IF PROFICIENT, R_i/R* IF NOT (WHERE R* IS THE RAW PROFICIENCY THRESHOLD);

#delimit;
gen prof=prof_level>=3;

gen temp=raw if prof==1; bysort yr subj grade: egen raw_prof=min(temp); drop temp;
gen index=min(raw/raw_prof,1);

*** COMPUTE RETURNS TO A CORRECT ANSWER TO EACH QUESTION FOR THE DIFFERENT TEST SCORE TRANSFORMATIONS (*_return)
*** - raw0 = RAW SCORE WHEN QUESTION q IS ANSWERED INCORRECTLY (SIMILARLY FOR scale0, tau0, prof0, index0)
*** - raw1 = RAW SCORE WHEN QUESTION q IS ANSWERED   CORRECTLY (SIMILARLY FOR scale1, tau1, prof1, index1)
*** - NOTE THAT tau1 OR tau0 MAY BE MISSING IF NO ONE EARNED A PARTICULAR RAW SCORE VALUE---THIS WON'T REALLY AFFECT OUR CALCULATIONS SINCE THESE ARE RARE OUTCOMES;

preserve;
#delimit;
keep   yr subj grade raw tau scale;
bysort yr subj grade raw tau scale: keep if _n==1;
foreach v in raw tau scale {; rename `v'  `v'0;}; save $st/tomerge0, replace;
foreach v in raw tau scale {; rename `v'0 `v'1;}; save $st/tomerge1, replace;
restore;

#delimit;
gen raw0=raw -u; merge m:1 yr subj grade raw0 using $st/tomerge0; drop if _merge==2; tab raw0 if _merge==1; drop _merge; erase $st/tomerge0.dta;
gen raw1=raw0+1; merge m:1 yr subj grade raw1 using $st/tomerge1; drop if _merge==2; tab raw1 if _merge==1; drop _merge; erase $st/tomerge1.dta;

bysort yr subj grade: egen min=min(tau  ); replace tau0  =min if tau0  ==.; replace tau1  =min if tau1  ==.; drop min;
by     yr subj grade: egen min=min(scale); replace scale0=min if scale0==.; replace scale1=min if scale1==.; drop min;

#delimit;
gen prof0=raw0>=raw_prof;
gen prof1=raw1>=raw_prof;

gen index0=min(raw0/raw_prof,1);
gen index1=min(raw1/raw_prof,1);

gen scale_return=scale1-scale0;
gen tau_return  =tau1  -tau0  ;
gen prof_return =prof1 -prof0 ;
gen index_return=index1-index0;

*** COMPUTE DERIVATIVE OF PROBABILITY OF A CORRECT ANSWER FOR EACH QUESTION (dp)
*** COMPUTE THE TOTAL SCORE RETURN TO AN INCREASE ABILITY FOR EACH TEST SCORE TRANSFORMATION (d_*)
*** NORMALIZE THE SCORE RETURNS TO REFLECT 1 CORRECT ANSWER ON A 50-QUESTION EXAM;

gen p =c+(1-c)/(1+exp(-$D*a*(theta-b)));
gen dp=$D*a*(1-c)*exp(-$D*a*(theta-b))/(1+exp(-$D*a*(theta-b)))^2;
gen I =dp^2/(p*(1-p));

do $cod/collapse-data.do;
